from logging import getLogger
from pathlib import Path

from .config import CONFIG as _CONFIG
from .util import create_temporary_docker_volume, run_docker_container


LOGGER = getLogger(__name__)
CONFIG = _CONFIG['ocr']


def run_tesseract(client, preprocessing_volume, second_pass_volume) -> None:
    with create_temporary_docker_volume(client) as first_pass_volume:
        run_tesseract_first_pass(client, preprocessing_volume, first_pass_volume)
        run_tesseract_second_pass(client, preprocessing_volume, first_pass_volume, second_pass_volume)


def run_tesseract_first_pass(client, preprocessing_volume, first_pass_volume) -> None:
    LOGGER.info('Running the first pass of Tesseract with all languages')

    volumes = {
        preprocessing_volume.name: {
            'bind': '/input',
            'mode': 'ro',
        },
        first_pass_volume.name: {
            'bind': '/output',
            'mode': 'rw',
        },
    }

    command = [
        'parallel', '--halt', 'now,fail=1', '--jobs', '25%', '--',
        f'nice -n {CONFIG.getint("niceness")} tesseract {{}} /output/{{/.}} -l {CONFIG["languages"]} '
        '--oem 1 --psm 3 --tessdata-dir /tessdata hocr', '::::', '/input/list.txt',
    ]
    run_docker_container(client, 'ahisto/tesseract', command=command, volumes=volumes)

    command = [
        'python', '-m', 'scripts.extract_detected_languages', '/input/list.txt', '/output',
        CONFIG["languages"], '/output/detected-languages.txt', '00',
    ]
    run_docker_container(client, 'ahisto/ocr-eval', command=command, volumes=volumes)


def run_tesseract_second_pass(client, preprocessing_volume, first_pass_volume,
                              second_pass_volume) -> None:
    LOGGER.info('Running the second pass of Tesseract with selected languages')

    volumes = {
        preprocessing_volume.name: {
            'bind': '/input',
            'mode': 'ro',
        },
        first_pass_volume.name: {
            'bind': '/first-pass',
            'mode': 'ro',
        },
        second_pass_volume.name: {
            'bind': '/output',
            'mode': 'rw',
        },
    }

    command = [
        'parallel', '--halt', 'now,fail=1', '--jobs', '25%', '-N', '2', '--',
        'if [ {2} = None ]; then echo | tee /output/{1/.}.{hocr,txt,tsv,box,page}; else '
        f'nice -n {CONFIG.getint("niceness")} tesseract {{1}} /output/{{1/.}} -l {{2}} '
        '--oem 1 --psm 3 --tessdata-dir /tessdata hocr txt tsv makebox; fi', '::::',
        '/first-pass/detected-languages.txt'
    ]
    run_docker_container(client, 'ahisto/tesseract', command=command, volumes=volumes)

    del volumes[first_pass_volume.name]

    command = [
        'parallel', '--halt', 'now,fail=1', '--jobs', '100%', '--',
        f'! grep -q . /output/{{/.}}.hocr || nice -n {CONFIG.getint("niceness")} ocr-transform '
        'hocr page /output/{/.}.hocr /output/{/.}.page', '::::', '/input/list.txt',
    ]
    run_docker_container(client, 'ahisto/ocr-fileformat', command=command, volumes=volumes)


def run_google_vision_ai(client, postprocessing_volume, google_vision_ai_volume) -> None:
    LOGGER.info('Running Google Vision AI')

    volumes = {
        preprocessing_volume.name: {
            'bind': '/input',
            'mode': 'ro',
        },
        google_vision_ai_volume.name: {
            'bind': '/output',
            'mode': 'rw',
        },
    }

    command = [
        'parallel', '--halt', 'now,fail=1', '--jobs', '100%', '--',
        f'nice -n {CONFIG.getint("niceness")} scripts/google-vision-annotate.sh '
        '/input/google-api-key.txt {} /output/{/.}.json /output/{/.}.error', '::::',
        '/input/list.txt',
    ]
    run_docker_container(client, 'ahisto/ocr-eval', command=command, volumes=volumes)


def combine_results(client, tesseract_volume, google_vision_ai_volume, output_volume):
    pass
